# -*- coding: utf-8 -*-
"""
CREATED
   @author: Morten Lybech Thøgersen at 2021-05-04

DESCRIPTION:
   Example of request and download of site-res and netcdf files for datasets within  EMDAPI-GLOBAL-ATLAS-SERVICE

USAGE:
   Run the script.

   For documentation on client-side configuration and parameter-documentation, please refer to the following wiki-pages:

   EMD-API GLobal Atlas Services: https://help.emd.dk/mediawiki/index.php?title=EMD-API_-_Global_Atlas_Services
   Parameter Description: https://help.emd.dk/mediawiki/index.php?title=GASP_Global

NOTES:
   N/A

REFERENCES
   N/A
"""

import emdapiglobalatlasservice
import warnings
import shutil
import os
import tempfile
warnings.filterwarnings('ignore')


if __name__ == "__main__":

    out_folder = tempfile.gettempdir()
    print("Output folder: ", out_folder)
    width_km = 20
    height_km = 20
    parameters = [ "resource-omni", "resource-sector", "siting-extremes", "siting-fatigue-omni", "siting-fatigue-sector", "turbine-fatigue", "turbine-fatigue-recommend", "turbine-extreme", "turbine-extreme-recommend" ]

    # Input-data for different sites
    storrotliden_dict = { "location_name": "StorRotliden-Sweden", "location_lon": 18.37, "location_lat": 64.23 }
    kassel_dict = { "location_name": "Kassel-Germany", "location_lon": 9.498, "location_lat": 51.312 }
    newsalem_dict = { "location_name": "NewSalem-USA", "location_lon": -101.4, "location_lat": 46.7 }
    florianopolis_dict = { "location_name": "Florinaopolis-Brasil", "location_lon": -48.6, "location_lat": -27.6 }
    lillgrunden_dict = { "location_name": "Lillgrunden-Sweden", "location_lon": 12.769718, "location_lat": 55.509097 }

    site_test_list = [lillgrunden_dict, storrotliden_dict, kassel_dict, newsalem_dict, florianopolis_dict ]

    # Login to EMDAPI-GLOBAL ATLAS SERVICES
    login_token = 'f079881f-4f1c-4193-b71e-1184d165eed7'  # Test-user access-token: To be replaced with your own token

    config_atlas = emdapiglobalatlasservice.Configuration()
    config_atlas.host = "https://api.emd.dk/global-atlas-service"
    config_atlas.api_key['X-Auth'] = login_token
    emdapi_atlas = emdapiglobalatlasservice.GlobalAtlasServicesApi(emdapiglobalatlasservice.ApiClient(config_atlas))

    # Get list of datasets
    atlas_list = emdapi_atlas.atlasdata_controller_get_atlasservices()

    # Check if siteres files are available for specific dataset - then download if exists

    for site in site_test_list:

        for atlas_item in atlas_list:

            print("")
            print("----------------------------------------------------------")
            print("EMD-API Global Atlas Service - Download Files. ")
            print("   Site: ", site["location_name"])
            print("   Dataset: ", atlas_item.name)
            print("   Service-id: ", atlas_item.service_id)
            print("   Documentation: ", atlas_item.url)
            print("----------------------------------------------------------")

            for outformat in atlas_item.output_types_areas:

                ok_continue = ("siteres" in atlas_item.output_types_areas) and ("netcdf" in atlas_item.output_types_areas)
                if not(ok_continue):
                    continue

                ok_coverage = emdapi_atlas.atlasdata_controller_get_atlascoveragepoint(atlas_item.service_id, site["location_lat"], site["location_lon"])
                if ok_coverage:

                    # Most parameters are "packed" into the siteres format - so we only order that once per site (but not the turbine classification ones, however that can be accessed separately, see below)
                    if outformat == "siteres":
                        try:

                            filename = os.path.join(out_folder, atlas_item.name+"_SITERES_"+site["location_name"]+"_w"+str(width_km)+"km_h"+str(height_km)+"km."+outformat)
                            print("  Siteres - ", filename)
                            if os.path.exists(filename):
                                os.remove(filename)
                            with emdapi_atlas.atlasdata_controller_get_atlasdownload_area(atlas_item.service_id, "resource-sector", site["location_lat"], site["location_lon"], width_km, height_km, data_type=outformat, _preload_content=False) as file_object:
                                with open(filename, 'wb') as out_file:
                                    shutil.copyfileobj(file_object, out_file)
                                file_object.release_conn()
                        except Exception as e:
                            print("Error: "+str(e))

                    # Getting all parameters (as netcdf)
                    if outformat == "netcdf":
                        try:
                            # Most parameters are "packed" into the siteres format - so we only order that once per site
                            for parameter in parameters:
                                filename = os.path.join(out_folder, atlas_item.name+"_"+parameter+"_"+site["location_name"]+"_w"+str(width_km)+"km_h"+str(height_km)+"km."+outformat)
                                print("  "+parameter+" - " + filename)
                                if os.path.exists(filename):
                                    os.remove(filename)
                                with emdapi_atlas.atlasdata_controller_get_atlasdownload_area(atlas_item.service_id, parameter, site["location_lat"], site["location_lon"], width_km, height_km, data_type=outformat, _preload_content=False) as file_object:
                                    with open(filename, 'wb') as out_file:
                                        shutil.copyfileobj(file_object, out_file)
                                    file_object.release_conn()
                        except Exception as e:
                            print("Error: "+str(e))


